import os

import toml


class PoetryManager(object):

    def __init__(self):
        self.path = os.getcwd().replace("tests", "") if 'tests' in os.getcwd() else os.getcwd()

    def get_version(self):
        os.system("poetry update && poetry install && poetry build")
        return self.load_toml()['tool']['poetry']['version']

    def load_toml(self, filename='pyproject.toml'):
        return toml.load(os.path.join(self.path, filename))

    def save_toml(self, tml, filename='pyproject.toml'):
        with open(os.path.join(self.path, filename), 'w') as f:
            f.write(toml.dumps(tml))



import traceback

import boto3

from baram.log_manager import LogManager


class EC2Manager(object):
    def __init__(self):
        self.cli = boto3.client('ec2')

        self.logger = LogManager.get_logger()

    def list_sgs(self) -> list:
        """
        Describes the specified security groups or all of your security groups.

        :return: SecurityGroups
        """
        try:
            return self.cli.describe_security_groups()['SecurityGroups']
        except:
            print(traceback.format_exc())
            return None

    def list_unused_sg_ids(self, description_filter: str = '', sm_domain_ids: list = None) -> set:
        """
        Describe useless and deletable security group ids.
        (including disused SageMaker domain related things)

        :param description_filter: String in 'Description'
        :param sm_domain_ids: DomainId of sagemaker
        :return: GroupIds
        """
        # Get security groups not related to anything in ec2
        sgs = self.list_sgs()
        try:
            valid_sg_ids = set([pair['sg_id'] for pair in self.list_vpc_sg_eni_subnets()])
            result = [sg['GroupId'] for sg in sgs
                      if sg['GroupId'] not in valid_sg_ids
                      or (description_filter in sg['Description']
                          and sum([domain_id in sg['Description'] for domain_id in sm_domain_ids]) == 0)]

            return set(result)
        except TypeError:
            return None

    def list_vpc_sg_eni_subnets(self) -> list:
        """
        Return the list of all pairs of {VpcId, GroupId, NetworkInterfaceId, SubnetId}.

        :return: The list of {vpc_id, security_group_id, eni_id, subnet_id}
        """
        result = []
        vpc_ids = [vpc['VpcId'] for vpc in self.list_vpcs() if vpc['State'] == 'available']

        try:
            for vpc_id in vpc_ids:
                specified_sg_ids = self.get_sg_ids_with_vpc_id(vpc_id=vpc_id)
                for sg_id in specified_sg_ids:
                    specified_enis = self.get_eni_with_sg_id(sg_id=sg_id)
                    pairs = {'vpc_id': vpc_id, 'sg_id': sg_id}
                    for eni in specified_enis:
                        pairs['eni_id'], pairs['subnet_id'] = eni['NetworkInterfaceId'], eni['SubnetId']
                    result.append(pairs)
            return result

        except TypeError:
            print(traceback.format_exc())
            return None

    def get_default_vpc(self) -> list:
        """
        Get default vpc

        :return: Default Vpc
        """
        return [vpc for vpc in self.list_vpcs() if vpc['IsDefault']][0]

    def get_sg_ids_with_vpc_id(self, vpc_id: str) -> list:
        """
        Get security group ids of specific vpc.

        :param vpc_id: VpcId
        :return: GroupId
        """
        sgs = self.cli.describe_security_groups()['SecurityGroups']
        try:
            return [sg['GroupId'] for sg in sgs if sg['VpcId'] == vpc_id]
        except TypeError:
            print(traceback.format_exc())
            return None

    def get_eni_with_sg_id(self, sg_id: str) -> list:
        """
        Get network interfaces with specific security group.

        :param sg_id: GroupId
        :return: NetworkInterfaces
        """
        enis = self.cli.describe_network_interfaces()['NetworkInterfaces']
        try:
            return [eni for eni in enis if eni['Groups'] != [] and sg_id in [x['GroupId'] for x in eni['Groups']]]
        except TypeError:
            print(traceback.format_exc())
            return None

    def list_sg_relations(self) -> list:
        """
        Describe security groups with related security groups, including egress and ingress status.

        :return: SecurityGroupId, IsEgress, RelatedSecurityGroupId
        """
        sg_rules = self.cli.describe_security_group_rules()['SecurityGroupRules']

        sg_ids = [sg_rule['GroupId'] for sg_rule in sg_rules]
        is_egress_list = [sg_rule['IsEgress'] for sg_rule in sg_rules]
        referenced_sgs = [x['ReferencedGroupInfo']['GroupId']
                          if 'ReferencedGroupInfo' in x.keys() else None for x in sg_rules]

        result = [{'sg_id': sg_ids[i],
                   'is_egress': is_egress_list[i],
                   'related_sg_id': referenced_sgs[i]} for i in range(len(sg_rules))]
        return result

    def get_related_sgs(self, sg_id: str) -> set:
        """
        Get the set of related security groups for specific security group.

        :param sg_id: GroupId
        :return: GroupId
        """
        sg_relations = self.list_sg_relations()
        return set([sg_relation['sg_id'] for sg_relation in sg_relations if sg_relation['related_sg_id'] == sg_id])

    def get_sg_rules(self, sg_id: str) -> list:
        """
        Get security group rules of specific security group.

        :param sg_id: GroupId
        :return: SecurityGroupRuleId
        """
        sg_rules = self.cli.describe_security_group_rules()['SecurityGroupRules']

        result = [{'sg_rule_id': sg_rule['SecurityGroupRuleId'],
                   'is_egress': True if sg_rule['IsEgress'] else False}
                  for sg_rule in sg_rules if sg_rule['GroupId'] == sg_id]
        return result

    def delete_sg_rules(self, sg_id: str):
        """
        Get rid of security group rule of specific security group.

        :param sg_id: GroupId
        """
        sg_rules = self.get_sg_rules(sg_id)

        try:
            for sg_rule in sg_rules:
                if sg_rule['is_egress']:
                    self.cli.revoke_security_group_egress(GroupId=sg_id,
                                                          SecurityGroupRuleIds=[sg_rule['sg_rule_id']])
                else:
                    self.cli.revoke_security_group_ingress(GroupId=sg_id,
                                                           SecurityGroupRuleIds=[sg_rule['sg_rule_id']])
                self.logger.info('security group rule has deleted')
        except:
            print(traceback.format_exc())

    def delete_sgs(self, sg_ids: list):
        """
        Delete useless and deletable security groups.

        :param sg_ids: List of GroupId.
        :return:
        """
        try:
            for sg_id in sg_ids:
                self.delete_sg_rules(sg_id)
                self.delete_sg(sg_id)
        except:
            print(traceback.format_exc())

    def delete_sg(self, sg_id: str):
        """
        Delete security group.

        :param sg_id: GroupId
        """
        try:
            self.cli.delete_sg(GroupId=sg_id)
            self.logger.info('security group has deleted')
        except:
            print(traceback.format_exc())

    def list_instances_with_status(self, status: str = 'running'):
        """
        Describes all instances in specific status (ex: 'running', ...)

        :return: Instances, with status
        """
        try:
            instances = [instance['Instances'][0] for instance in self.describe_instances()['Reservations']]
            return [instance for instance in instances if instance['State']['Name'] == status]
        except:
            print(traceback.format_exc())
            return None

    def delete_unused_key_pairs(self):
        """
        Delete redundant key pairs (i.e. not related to any instances)
        """
        key_pairs_redundant = self.list_unused_key_pairs()
        try:
            for key_pair in key_pairs_redundant:
                self.cli.delete_key_pair(KeyName=key_pair)
                self.logger.info('key pair has deleted')
        except:
            print(traceback.format_exc())

    def list_unused_key_pairs(self):
        """
        Describes all disused key pairs

        :return: KeyName
        """
        key_pairs_total = self.list_key_pairs()
        instances = [instance['Instances'][0] for instance in self.describe_instances()['Reservations']]
        key_pairs_using = [instance['KeyName'] for instance in instances if 'KeyName' in instance]

        return key_pairs_total - set(key_pairs_using)

    def list_key_pairs(self):
        """
        Describes all key pairs

        :return: KeyName
        """
        key_pairs = self.cli.describe_key_pairs()['KeyPairs']
        return set([key_pair['KeyName'] for key_pair in key_pairs])

    def list_vpcs(self) -> list:
        """
        List one or more of your VPCs.

        :return: Vpcs
        """
        return self.cli.describe_vpcs()['Vpcs']

    def list_detail_vpcs(self) -> list:
        """
        List one or more of your VPCs with detail.

        :return: Vpcs.
        """
        vpc_list = []
        for vpc in self.list_vpcs():
            name = ''
            if 'Tags' in vpc:
                name = next(t['Value'] for t in vpc['Tags'] if t['Key'] == 'Name')
            elif vpc['IsDefault']:
                name = 'default'
            vpc_list.append(
                {'vpc_id': vpc['VpcId'],
                 'name': name,
                 'cidr_block': vpc['CidrBlock'],
                 'state': vpc['State']})
        return vpc_list

    def list_subnets(self) -> list:
        """
        List one or more of your Subnets.

        :return: Subnets
        """
        return self.cli.describe_subnets()['Subnets']

    def list_detail_subnets(self) -> list:
        """
        List one or more of your Subnets with detail.

        :return: Subnets
        """
        subnet_list = []
        for subnet in self.list_subnets():
            name = ''
            vpc_name = ''
            if 'Tags' in subnet:
                name = [t['Value'] for t in subnet['Tags'] if t['Key'] == 'Name']
                vpc_name = [t['Value'] for t in subnet['Tags'] if t['Key'] == 'aws:cloudformation:stack-name']

            subnet_list.append(
                {'subnet_id': subnet['SubnetId'],
                 'vpc_id': subnet['VpcId'],
                 'name': '' if len(name) == 0 else name[0],
                 'vpc_name': '' if len(vpc_name) == 0 else vpc_name[0],
                 'cidr_block': subnet['CidrBlock'],
                 'state': subnet['State']})
        return subnet_list

    def get_subnet_ids_with_vpc_id(self, vpc_id: str) -> list:
        """
        Retrieve subnet ids attached to specified VPC id
        :param vpc_id: vpc id
        :return:
        """
        return [i['SubnetId'] for i in self.list_subnets() if i['VpcId'] == vpc_id]

    def list_enis(self) -> list:
        """
        List one or more of your Network Interfaces.

        :return:
        """
        return self.cli.describe_network_interfaces()['NetworkInterfaces']

    def list_tgws(self) -> list:
        """
        Get Id of Transit Gateway

        :return: TransitGateways
        """
        return self.cli.describe_transit_gateways()['TransitGateways']

    def get_sg_id_with_sg_name(self, group_name: str) -> str:
        """
        Retrieve subnet id from group name.

        :param group_name: GroupName
        :return: GroupId
        """
        return next(
            (i['GroupId'] for i in self.cli.describe_security_groups()['SecurityGroups']
             if group_name.lower() in i['GroupName'].lower()), None)

    def get_vpc_id_with_vpc_name(self, vpc_name: str) -> str:
        """
        Retrieve vpc id from vpc name

        :param vpc_name: vpc name
        :return:
        """
        return next(i['VpcId'] for i in self.list_vpcs() if 'Tags' in i
                    for t in i['Tags'] if vpc_name.lower() in t['Value'].lower())

    def get_subnet_id(self, vpc_id: str, subnet_name: str) -> str:
        """
        Retrieve subnet id from vpc id and subnet name.

        :param vpc_id: vpc_id
        :param subnet_name: subnet_name
        :return:
        """
        return next(s['SubnetId'] for s in self.list_subnets() if vpc_id == s['VpcId'] and 'Tags' in s
                    for t in s['Tags'] if subnet_name == t['Value'])

    def get_ec2_id_with_ec2_name(self, ec2_name: str) -> str:
        """

        :param ec2_name: ec2 instance name
        :return:
        """
        ec2 = boto3.resource('ec2')
        return next(
            i.id for i in ec2.instances.all() if i.state['Name'] == 'running' for t in i.tags if ec2_name == t['Value'])

    def describe_instances(self, instance_id_list: list = None) -> list:
        """

        Retrieve ec2 instance description.
        :param instance_id_list: ec2 instance id list
        :return:
        """
        if instance_id_list is not None:
            return self.cli.describe_instances(InstanceIds=instance_id_list)
        else:
            return self.cli.describe_instances()

    def get_ec2_instances_with_imds_v1(self) -> list:
        """

        :return: get ec2 instances that support imds_v1.
        """
        response = self.describe_instances()
        return [i['InstanceId'] for r in response['Reservations']
                for i in r['Instances']
                if i['MetadataOptions']['HttpTokens'] != 'required' and i['State']['Name'] == 'running']

    def apply_imdsv2_only_mode(self,
                               instances_list: list = None,
                               http_put_response_hop_limit: int = 1):
        """

        Apply imdsv2 only mode into ec2 instances.
        :param instances_list:
        :param http_put_response_hop_limit: see https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_InstanceMetadataOptionsRequest.html.
        :return:
        """
        for i in instances_list:
            self.cli.modify_instance_metadata_options(InstanceId=i,
                                                      HttpTokens='required',
                                                      HttpPutResponseHopLimit=http_put_response_hop_limit,
                                                      HttpEndpoint='enabled')

    def delete_vpc(self, vpc_id: str):
        """
        Delete vpc with vpc_id.

        :param vpc_id:
        :return:
        """
        pass
        # TODO: TBD.
        # delete vpc with SG, EP and Key Pair.



import boto3


class KMSManager(object):
    def __init__(self):
        self.cli = boto3.client('kms')

    def list_keys(self):
        '''
        Gets a list of all KMS keys in the caller's Amazon Web Services account and Region.

        :return: KMS keys.
        '''
        return self.cli.list_keys()

    def list_aliases(self):
        '''
        Gets a list of aliases in the caller's Amazon Web Services account and region.

        :return: KMS aliases.
        '''
        return self.cli.list_aliases()

    def get_kms_arn(self, alias_name: str, arn_or_key: bool = True):
        '''
        Retrieve kms arn by alias.

        :param alias_name: alias name.
        :param arn_or_key: True returns ARN, Flase returns key only.
        :return: arn or KMS key id.
        '''

        alias_name = f'alias/{alias_name}' if not 'alias/' in alias_name else alias_name
        for k in self.list_aliases()['Aliases']:
            if alias_name == k['AliasName']:
                return k['AliasArn'].replace(alias_name, f'key/{k["TargetKeyId"]}') if arn_or_key else k["TargetKeyId"]



import os
import tempfile

import boto3

from baram.s3_manager import S3Manager
from baram.log_manager import LogManager
from baram.process_manager import ProcessManager


class LambdaManager(object):
    def __init__(self):
        self.cli = boto3.client('lambda')
        self.logger = LogManager.get_logger('LambdaManager')

    def list_layers(self):
        '''
        Lists Lambda layers and shows information about the latest version of each.

        :return: list layers
        '''
        return self.cli.list_layers()['Layers']

    def get_latest_layer_arn(self, layer_name):
        '''
        Retrieve latest layer arn from layer name.

        :param layer_name: layer name
        :return: baram latest layer ARN.
        '''
        return [l['LatestMatchingVersion']['LayerVersionArn']
                for l in self.list_layers() if l['LayerName'] == layer_name][0]

    def publish_lambda_layer(self, layer_name, s3_bucket_name: str, sm: S3Manager):
        '''
        publish lamber layer from local wheel.

        :param layer_name: layer name
        :param s3_bucket_name: s3 bucket name
        :param sm: S3Manager instance
        :return:
        '''
        temp_dir = tempfile.mkdtemp()
        os.mkdir(os.path.join(temp_dir, 'python'))
        zip_path = os.path.join(tempfile.mkdtemp(), f'{layer_name}.zip')
        cmd = f'cd {temp_dir} && pip3 install {layer_name} -t python ' \
              f'&& zip -r {zip_path} python '
        self.logger.info(cmd)
        ProcessManager.run_cmd(cmd, False)

        sm.upload_file(zip_path, f'lambda_layers/{layer_name}.zip')

        content = {
            'S3Bucket': s3_bucket_name,
            'S3Key': f'lambda_layers/{layer_name}.zip'
        }
        self.logger.info(content['S3Key'])

        arn = ':'.join(self.get_latest_layer_arn(layer_name).split(":")[:-1])
        self.cli.publish_layer_version(LayerName=arn,
                                       Content=content,
                                       CompatibleRuntimes=['python3.7', 'python3.8', 'python3.9'])



import boto3


class ECRManager(object):
    def __init__(self):
        self.cli = boto3.client('ecr')

    def describe_images(self, name: str, max_result: int = 100):
        '''

        :param name: repo name
        :param max_result: default 100
        :return:
        '''
        response = self.cli.describe_images(
            repositoryName=name,
            maxResults=max_result,
        )
        return response['imageDetails']

    def list_images(self, name: str, max_result: int = 100):
        '''

        :param name: repo name
        :param max_result: default 100
        :return:
        '''
        response = self.cli.list_images(
            repositoryName=name,
            maxResults=max_result
        )
        return response['imageIds']



import boto3
import traceback

from baram.log_manager import LogManager


class EFSManager(object):
    def __init__(self):
        self.cli = boto3.client('efs')
        self.logger = LogManager.get_logger()

    def list_efs(self):
        """
        List one or more of your EFS.

        :return:
        """
        return self.cli.describe_file_systems()['FileSystems']

    def list_mount_targets(self, efs_id: str):
        """
        List mount targets of specific file system.

        :param efs_id: FileSystemId
        :return:
        """
        try:
            return self.cli.describe_mount_targets(FileSystemId=efs_id)['MountTargets']
        except:
            print(traceback.format_exc())
            return None

    def list_redundant_efs(self, redundant_sm_domain_ids: list = []):
        """
        Describe redundant file systems

        :param redundant_sm_domain_ids: DomainId
        :return: List of FileSystemId
        """
        # TODO: Need more cases (sagemaker case only)
        redundant_efs_ids = [efs['FileSystemId'] for efs in self.list_efs()
                             if efs['CreationToken'] not in redundant_sm_domain_ids
                             and 'sagemaker' in efs['Tags'][0]['Value']]
        return redundant_efs_ids

    def delete_mount_targets(self, mount_target_id: str):
        """
        Delete mount targets via its id.

        :param mount_target_id: MountTargetId
        :return:
        """
        try:
            self.cli.delete_mount_target(MountTargetId=mount_target_id)
        except:
            print(traceback.format_exc())

    def delete_efs(self, efs_id: str):
        """
        Delete specific file system.

        :param efs_id: FileSystemId
        :return:
        """
        try:
            mount_target_ids = [mt['MountTargetId'] for mt in self.list_mount_targets(efs_id)]

            for mt_id in mount_target_ids:
                self.delete_mount_targets(mt_id)

            is_mount_targets_remain = (len(self.list_mount_targets(efs_id)) != 0)
            while is_mount_targets_remain:
                is_mount_targets_remain = (len(self.list_mount_targets(efs_id)) != 0)

            if is_mount_targets_remain:
                self.cli.delete_efs(FileSystemId=efs_id)

            self.logger.info('efs has deleted')
        except:
            print(traceback.format_exc())



import asyncio

import aiohttp
import nest_asyncio


class AsyncCrawler(object):

    def request_urls(self, method: str, urls: list, **kwargs: dict) -> list:
        '''
        Retruns crawled htmls from urls using async io.

        :param method: GET or POST
        :param urls: http url list
        :param **kwargs: additional parameters.
        :return: html list
        '''

        try:
            loop = asyncio.get_event_loop()
        except RuntimeError:
            nest_asyncio.apply()
        finally:
            return loop.run_until_complete(self._fetch_pages(method, urls, **kwargs))

    async def _fetch_pages(self, method: str, urls: list, **kwargs: dict):
        async with aiohttp.ClientSession() as session:
            return await asyncio.gather(
                *[self._fetch_page(session, method, url, **kwargs) for url in urls], return_exceptions=True)

    async def _fetch_page(self, session, method: str, url: str, **kwargs: dict) -> str:
        async with session.request(method, url, ssl=False, **kwargs) as response:
            html = await response.text()
            return html if response.status == 200 else None



import fire
import boto3

from baram.log_manager import LogManager


class IAMManager(object):
    def __init__(self):
        self.cli = boto3.client('iam')
        self.logger = LogManager.get_logger()

    def get_role(self, role_name):
        '''
        Retrieves information about the specified role.

        :param role_name: role name
        :return:
        '''
        return self.cli.get_role(RoleName=role_name)['Role']

    def get_role_arn(self, role_name):
        '''
        Retrieves role ARN.

        :param role_name: role name
        :return:
        '''
        return self.get_role(role_name)['Arn']

    def list_role_policies(self, role_name):
        '''
        Lists the names of the inline policies that are embedded in the specified IAM role.

        :param role_name: role name
        :return:
        '''
        return self.cli.list_attached_role_policies(RoleName=role_name)['AttachedPolicies']

    def attach_group_policy(self, group_name, policy_arn):
        '''
        Attaches the specified managed policy to the specified IAM group.

        :param group_name: group name
        :param policy_arn: policy arn
        :return:
        '''
        return self.cli.attach_group_policy(GroupName=group_name, PolicyArn=policy_arn)

    def detach_group_policy(self, group_name, policy_arn):
        '''
        Removes the specified managed policy from the specified IAM group.

        :param group_name: group name
        :param policy_arn: policy arn
        :return:
        '''
        return self.cli.detach_group_policy(GroupName=group_name, PolicyArn=policy_arn)

    def list_group_policies(self, user_group_name, max_items=100):
        '''
        Lists the names of the inline policies that are embedded in the specified IAM group.

        :param user_group_name: user group name
        :param max_items: max items, default=100
        :return:
        '''
        return self.cli.list_group_policies(GroupName=user_group_name, MaxItems=max_items)

    def list_policies(self, scope: str = 'All', max_result: int = 1000):
        """
        Lists policies in IAM
        :param scope: 'All' for all policies, 'Local' for customer managed policies, 'AWS' for AWS managed policies
        :param max_result: max number of results (max=1000)
        :return:
        """
        policies = self.cli.list_policies(Scope=scope, MaxItems=max_result)
        result = policies['Policies']
        while 'Marker' in policies:
            policies = self.cli.list_policies(Scope=scope, MaxItems=max_result, Marker=policies['Marker'])
            result += policies['Policies']
        return result

    def list_redundant_policies(self, scope: str = 'Local'):
        """
        Lists redundant policies that are not attached to any IAM user, group, or role
        :param scope: 'All' for all policies, 'Local' for customer managed policies, 'AWS' for AWS managed policies
        :return:
        """
        return [i for i in self.list_policies(scope=scope) if i['AttachmentCount'] == 0]


if __name__ == '__main__':
    fire.Fire(IAMManager)



import sys
import logging


class LogManager(object):

    @staticmethod
    def get_logger(name=''):
        logger = logging.getLogger(name)
        logger.propagate = False
        logger.setLevel(logging.INFO)

        streamHandler = logging.StreamHandler(sys.stdout)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        streamHandler.setFormatter(formatter)
        logger.addHandler(streamHandler)

        return logger



import time
from subprocess import Popen, PIPE, STDOUT


class ProcessManager(object):

    @classmethod
    def run_cmd(cls, cmd, sleep=True):
        '''
        run shell command with standard output.

        :param cmd: shell command
        :param sleep: sleeps a second or not.
        :return:
        '''
        p = Popen(cmd, stdout=PIPE, stderr=STDOUT, shell=True)

        while True:
            line = p.stdout.readline()
            print(line.decode('utf-8'))
            if not line: break
            if sleep:
                time.sleep(1)



import boto3


class AirflowManager(object):
    def __init__(self):
        self.cli = boto3.client('mwaa')

    def get_environment(self, name: str):
        '''
        Describes an Amazon Managed Workflows for Apache Airflow (MWAA) environment.

        :param name: environment name
        :return: Environment response
        '''
        return self.cli.get_environment(Name=name)['Environment']

    def update_environment(self, name: str):
        '''
        Updates an Amazon Managed Workflows for Apache Airflow (MWAA) environment.

        :param name: environment name
        :return:
        '''
        self.cli.update_environment(Name=name)



import os
from pathlib import Path
from pprint import pprint

import boto3
import fire

from baram.s3_manager import S3Manager
from baram.iam_manager import IAMManager
from baram.log_manager import LogManager


class GlueManager(object):
    def __init__(self, s3_bucket_name: str, table_path_prefix='table'):
        '''

        :param s3_bucket_name: s3 bucket name where Glue uses as default.
        '''

        self.logger = LogManager.get_logger()
        self.cli = boto3.client('glue')
        self.im = IAMManager()

        self.worker_type = 'G.1X'
        self.workers_num = 2
        self.timeout = 2880
        self.max_concurrent_runs = 123
        self.max_retries = 0
        self.python_ver = '3'
        self.glue_ver = '3.0'
        self.s3_path = f's3://{s3_bucket_name}'
        self.sm = S3Manager(s3_bucket_name)
        self.TABLE_PATH_PREFIX = table_path_prefix

        # See https://docs.aws.amazon.com/glue/latest/dg/aws-glue-programming-etl-glue-arguments.html
        self.default_args = {
            '--job-language': 'scala',
            '--TempDir': os.path.join(self.s3_path, 'temp'),
            '--enable-continuous-cloudwatch-log': 'true',
            '--enable-glue-datacatalog': 'true',
            '--enable-job-insights': 'true',
            '--enable-metrics': 'true',
            '--enable-spark-ui': 'true',
            '--job-bookmark-option': 'job-bookmark-enable',
            '--spark-event-logs-path': os.path.join(self.s3_path, 'events/'),
            '--encryption-type': 'sse-kms'
        }

    def start_job_run(self, name: str):
        '''

        :param name: job name
        :return:
        '''
        return self.cli.start_job_run(
            JobName=name
        )

    def _get_command(self, name: str):
        '''

        :param name: get command object when you create or update glue job.
        :return:
        '''

        return {
            'Name': 'glueetl',
            'ScriptLocation': os.path.join(self.s3_path, "scripts", f'{name}.scala'),
            'PythonVersion': self.python_ver
        }

    def create_job(self,
                   name: str,
                   package_name: str,
                   role_name: str,
                   extra_jars: str,
                   security_configuration: str):
        '''

        :param name: glue job name
        :param package_name: glue jar package name
        :param role_name:  role name
        :param extra_jars: extra jar path in s3
        :param security_configuration:  security configuration
        :return:
        '''

        self.default_args['--class'] = f'{package_name}.{name}'
        self.default_args['--extra-jars'] = extra_jars

        try:
            self.cli.create_job(
                Name=name,
                Description='',
                Role=self.im.get_role_arn(role_name),
                ExecutionProperty={
                    'MaxConcurrentRuns': self.max_concurrent_runs
                },
                Command=self._get_command(name),
                DefaultArguments=self.default_args,
                MaxRetries=self.max_retries,
                Timeout=self.timeout,
                SecurityConfiguration=security_configuration,
                GlueVersion=self.glue_ver,
                NumberOfWorkers=self.workers_num,
                WorkerType=self.worker_type
            )
        except self.cli.exceptions.IdempotentParameterMismatchException as e:
            self.logger.error(str(e))

    def get_job(self, job_name: str):
        '''

        :param job_name: glue job name.
        :return: glue job
        '''
        return self.cli.get_job(JobName=job_name)

    def update_job(self,
                   name: str,
                   package_name: str,
                   role_name: str,
                   extra_jars: str,
                   security_configuration: str):
        '''

        :param name: job name
        :param package_name: glue jar package name
        :param role_name:  role name
        :param extra_jars: extra jar path in s3
        :param security_configuration:  security configuration
        :return:
        '''

        self.default_args['--class'] = f'{package_name}.{name}'
        self.default_args['--extra-jars'] = extra_jars

        return self.cli.update_job(
            JobName=name,
            JobUpdate={
                'Role': self.im.get_role_arn(role_name),
                'ExecutionProperty': {
                    'MaxConcurrentRuns': self.max_concurrent_runs
                },
                'Command': self._get_command(name),
                'DefaultArguments': self.default_args,
                'MaxRetries': self.max_retries,
                'Timeout': self.timeout,
                'WorkerType': self.worker_type,
                'NumberOfWorkers': self.workers_num,
                'SecurityConfiguration': security_configuration,
                'GlueVersion': self.glue_ver
            }
        )

    def delete_job(self, name: str):
        '''

        :param name: job name
        :return:
        '''
        return self.cli.delete_job(JobName=name)

    def delete_table(self, db_name: str, table_name: str, include_s3: bool = False):
        '''

        :param db_name: database name
        :param table_name: job name
        :param include_s3: delete table including s3 or not
        :return:
        '''
        try:
            self.cli.delete_table(
                DatabaseName=db_name,
                Name=table_name
            )
        except Exception as e:
            pass
        finally:
            if include_s3:
                print(f'delete {os.path.join(self.TABLE_PATH_PREFIX, db_name, table_name)}')
                self.sm.delete_dir(os.path.join(self.TABLE_PATH_PREFIX, db_name, table_name))

    def get_table(self, db_name: str, table_name: str):
        '''

        :param db_name: database name
        :param table_name: table name
        :return:
        '''
        return self.cli.get_table(
            DatabaseName=db_name,
            Name=table_name
        )

    def refresh_job(self,
                    code_path: str,
                    exclude_names: list,
                    package_name: str,
                    role_name: str,
                    extra_jars: str,
                    security_configuration: str):
        '''

        :param code_path: code path
        :param exclude_names: job names to be excluded
        :param package_name: glue jar package name
        :param role_name: glue role name
        :param extra_jars: extra jar path in s3
        :param security_configuration: security configuraton
        :return:
        '''

        response = self.cli.list_jobs(MaxResults=1000)
        glue_jobs = set([f'{jn}.scala' for jn in response['JobNames']])
        git_jobs = set([f for f in os.listdir(code_path)])

        rest_in_glue = glue_jobs - git_jobs
        for f in rest_in_glue:
            name = Path(f).stem
            self.delete_job(name)
            self.logger.info(f'{name} deleted.')

        rest_in_git = git_jobs - glue_jobs
        for f in rest_in_git:
            name = Path(f).stem
            if name in exclude_names:
                continue
            self.create_job(name, package_name, role_name, extra_jars, security_configuration)
            self.logger.info(f'{name} created.')

    def summary(self):
        jobs = self.cli.list_jobs()
        if 'JobNames' in jobs:
            for j in jobs['JobNames']:
                r = self.cli.get_job_runs(JobName=j)
                if 'JobRuns' in r and len(r['JobRuns']) > 0:
                    last_run = r['JobRuns'][0]
                    last_run['StartedOn']

                    duration = int((last_run['CompletedOn'] - last_run['StartedOn']).total_seconds())
                    print(
                        f"{last_run['JobName']}\t{last_run['AllocatedCapacity']}\t{last_run['StartedOn']}\t{last_run['CompletedOn']}\t{duration}")


if __name__ == '__main__':
    fire.Fire(GlueManager)



import time

import boto3

from baram.log_manager import LogManager


class SagemakerManager(object):
    def __init__(self, domain_id: str = None):
        self.cli = boto3.client('sagemaker')
        self.domain_id = domain_id
        self.logger = LogManager.get_logger('SagemakerManager')

    def describe_user_profile(self, user_profile_name):
        response = self.cli.describe_user_profile(DomainId=self.domain_id, UserProfileName=user_profile_name)
        return response

    def list_apps(self, user_profile_name):
        response = self.cli.list_apps(DomainIdEquals=self.domain_id,
                                      UserProfileNameEquals=user_profile_name,
                                      SortBy='CreationTime',
                                      SortOrder='Descending',
                                      MaxResults=100)
        return response['Apps']

    def delete_app(self, user_profile_name, app_name, app_type):
        try:
            response = self.cli.delete_app(DomainId=self.domain_id,
                                           UserProfileName=user_profile_name,
                                           AppName=app_name,
                                           AppType=app_type)
            return response
        except:
            return None

    def describe_app(self, user_profile_name, app_name, app_type):
        response = self.cli.describe_app(DomainId=self.domain_id,
                                         UserProfileName=user_profile_name,
                                         AppName=app_name,
                                         AppType=app_type)
        return response

    def delete_user(self, user_profile_name):
        try:
            self.describe_user_profile(user_profile_name)
        except self.cli.exceptions.ResourceNotFound:
            self.logger.info(f'user {user_profile_name} does not exist.')
            return

        self.logger.info(f'list apps from {user_profile_name}')
        apps = self.list_apps(user_profile_name)
        for app in apps:
            try:
                response = self.describe_app(user_profile_name, app['AppName'], app['AppType'])
                if response['Status'] != 'Deleted' and response['Status'] != 'Deleting':
                    self.delete_app(user_profile_name, app['AppName'], app['AppType'])
            except self.cli.exceptions.ResourceNotFound:
                pass
            except self.cli.exceptions.ResourceInUse as e:
                self.logger.info(e)
                return
        self.logger.info(f'deleting {len(apps)} apps.')
        delete_cnt = 0
        elapsed_secs = 0
        while delete_cnt < len(apps):
            delete_cnt = 0
            for app in apps:
                response = self.describe_app(user_profile_name, app['AppName'], app['AppType'])
                self.logger.info(f'status = {response["Status"]}')
                if response['Status'] == 'Deleted' or response['Status'] == 'Failed':
                    delete_cnt += 1
            time.sleep(5)
            elapsed_secs += 5
            self.logger.info(f'wait 5 seconds. delete_cnt={delete_cnt}, elapsed_secs={elapsed_secs}')
        return self.cli.delete_user_profile(DomainId=self.domain_id, UserProfileName=user_profile_name)

    def list_domains(self):
        return self.cli.list_domains()['Domains']

    def delete_domain(self):
        response = self.cli.delete_domain(DomainId=self.domain_id, RetentionPolicy={'HomeEfsFileSystem': 'Delete'})
        return response

    def create_domain(self,
                      domain_name: str,
                      execution_role_name: str,
                      sg_group: str,
                      s3_kms_id: str,
                      efs_kms_id:str,
                      s3_output_path: str,
                      vpc_id:str,
                      subnet_id1:str,
                      subnet_id2:str,
                      instance_type: str = 'ml.t3.micro'):
        pass
        # TODO: TBD.
        response = self.cli.create_domain(
            DomainName=domain_name,
            AuthMode='IAM',
            DefaultUserSettings={
                'ExecutionRole': execution_role_name,
                'SecurityGroups': [
                    sg_group,
                ],
                'SharingSettings': {
                    'NotebookOutputOption': 'Allowed',
                    'S3OutputPath': s3_output_path,
                    'S3KmsKeyId': s3_kms_id
                },
                'JupyterServerAppSettings': {
                    'DefaultResourceSpec': {
                        'SageMakerImageArn': 'string',
                        'SageMakerImageVersionArn': 'string',
                        'InstanceType': instance_type
                    }
                },
                'KernelGatewayAppSettings': {
                    'DefaultResourceSpec': {
                        'SageMakerImageArn': 'string',
                        'SageMakerImageVersionArn': 'string',
                        'InstanceType': instance_type
                    },
                    'CustomImages': [
                        {
                            'ImageName': 'string',
                            'ImageVersionNumber': 1,
                            'AppImageConfigName': 'string'
                        },
                    ],
                    'LifecycleConfigArns': [
                        'string',
                    ]
                }
            },
            SubnetIds=[
                'string',
            ],
            VpcId=vpc_id,
            Tags=[
                {
                    'Key': 'string',
                    'Value': 'string'
                },
            ],
            AppNetworkAccessType='VpcOnly',
            KmsKeyId=efs_kms_id,
            AppSecurityGroupManagement='Service' | 'Customer',
            DomainSettings={
                'SecurityGroupIds': [
                    'string',
                ]
            }
        )

    def describe_image(self, image_name):
        try:
            return self.cli.describe_image(ImageName=image_name)
        except self.cli.exceptions.ResourceNotFound:
            self.logger.info('ResourceNotFound')
            return None

    def describe_image_version(self, image_name):
        try:
            return self.cli.describe_image_version(ImageName=image_name)
        except self.cli.exceptions.ResourceNotFound:
            self.logger.info('ResourceNotFound')
            return None

    def create_image_version(self, image_uri: str, name: str):
        return self.cli.create_image_version(
            BaseImage=image_uri,
            ImageName=name
        )

    def delete_image(self, image_name):
        try:
            return self.cli.delete_image(ImageName=image_name)
        except self.cli.exceptions.ResourceNotFound:
            self.logger.info('ResourceNotFound')
            return None

    def delete_image_version(self, image_name, version):
        try:
            return self.cli.delete_image_version(ImageName=image_name, Version=version)
        except self.cli.exceptions.ResourceNotFound:
            self.logger.info('ResourceNotFound')
            return None



import os

import boto3
from botocore.client import Config

from baram.log_manager import LogManager
from baram.kms_manager import KMSManager


class S3Manager(object):
    def __init__(self, bucket_name):
        self.cli = boto3.client('s3', config=Config(signature_version='s3v4'))
        self.km = KMSManager()
        self.logger = LogManager.get_logger('S3Manager')
        self.bucket_name = bucket_name
        try:
            bi = self.get_bucket_encryption()
            self.kms_algorithm, self.kms_id = bi['SSEAlgorithm'], bi['KMSMasterKeyID']
        except:
            self.kms_algorithm, self.kms_id = None, None

    def list_buckets(self):
        '''
        :return: response
        '''
        response = self.cli.list_buckets()
        return response['Buckets'] if 'Buckets' in response else None

    def put_object(self, s3_key_id: str, body):
        '''

        :param s3_key_id: s3 key id. ex) nylon-detector/a.csv
        :param body: byte or str data
        :return: response
        '''
        kwargs = {"Bucket": self.bucket_name,
                  "Key": s3_key_id,
                  "Body": body}
        if self.kms_id:
            kwargs['ServerSideEncryption'] = self.kms_algorithm
            kwargs['SSEKMSKeyId'] = self.kms_id
        response = self.cli.put_object(**kwargs)
        return response

    def get_object(self, s3_key_id: str):
        '''

        :param s3_key_id: s3 key id. ex) nylon-detector/a.csv
        :return: response
        '''
        try:
            response = self.cli.get_object(Bucket=self.bucket_name,
                                           Key=s3_key_id)
            return response['Body'].read()
        except self.cli.exceptions.NoSuchKey:
            self.logger.info(f'{s3_key_id} does not exist.')
            return None

    def get_object_by_lines(self, s3_key_id: str):
        '''
        get s3 object line by line.

        :param s3_key_id: s3 key id. ex) nylon-detector/a.csv
        :return: response
        '''

        try:
            import codecs
            line_stream = codecs.getreader("utf-8")
            response = self.cli.get_object(Bucket=self.bucket_name,
                                           Key=s3_key_id)
            return line_stream(response['Body'])
        except self.cli.exceptions.NoSuchKey:
            self.logger.info(f'{s3_key_id} does not exist.')
            return None

    def delete_object(self, s3_key_id: str):
        '''

        :param s3_key_id: s3 key id. ex) nylon-detector/a.csv
        :return: response
        '''
        return self.cli.delete_object(Bucket=self.bucket_name,
                                      Key=s3_key_id)

    def delete_objects(self, s3_keys: list, quiet: bool = True):
        objects = [{'Key': k} for k in s3_keys]
        return self.cli.delete_objects(Bucket=self.bucket_name,
                                       Delete={
                                           'Objects': objects,
                                           'Quiet': quiet
                                       })

    def upload_dir(self, local_dir_path: str, s3_dir_path: str):
        '''
        Upload directory.
        :param local_dir_path: local dir path. ex) /Users/lks21c/repo/sli-aflow
        :param s3_dir_path: s3 path. ex) nylon-detector/crawl_data
        :return: response
        '''
        self.logger.info('Uploading results to s3 initiated...')
        self.logger.info(f'local_path:{local_dir_path}, s3_path:{s3_dir_path}')
        try:
            for path, subdirs, files in os.walk(local_dir_path):
                for file in files:
                    dest_path = path.replace(local_dir_path, '')
                    s3file_path = os.path.normpath(s3_dir_path + '/' + dest_path + '/' + file)
                    local_file_path = os.path.join(path, file)

                    extra_args = {'ServerSideEncryption': self.kms_algorithm,
                                  'SSEKMSKeyId': self.kms_id} if self.kms_id else None
                    self.cli.upload_file(local_file_path,
                                         self.bucket_name,
                                         s3file_path,
                                         ExtraArgs=extra_args)
                    self.logger.info(
                        f'upload : {local_file_path} to Target: s3://{self.bucket_name}/{s3file_path} Success.')
        except Exception as e:
            self.logger.info(e)
            raise e

    def upload_file(self, local_file_path: str, s3file_path: str):
        '''
        Upload file.
        :param local_file_path: local file path. ex) /Users/lks21c/repo/sli-aflow/a.csv
        :param s3_dir_path: s3 path. ex) nylon-detector/crawl_data/a.csv
        :return: response
        '''

        try:
            extra_args = {'ServerSideEncryption': self.kms_algorithm,
                          'SSEKMSKeyId': self.kms_id} if self.kms_id else None
            self.cli.upload_file(local_file_path,
                                 self.bucket_name,
                                 s3file_path,
                                 ExtraArgs=extra_args)
            self.logger.info(f'upload : {local_file_path} to Target: s3://{self.bucket_name}/{s3file_path} Success.')
        except Exception as e:
            self.logger.info(e)
            raise e

    def download_dir(self, s3_dir_path: str, local_dir_path: str = os.getcwd()):
        '''
        Download directory from s3.

        :param s3_dir_path: s3 path. ex) nylon-detector/crawl_data
        :param local_dir_path: local dir path. ex) /Users/lks21c/repo/sli-aflow
        :return: response
        '''
        self.logger.info('Downloading results to s3 initiated...')
        self.logger.info(f's3_path:{s3_dir_path}, local_path:{local_dir_path}')
        bucket = boto3.resource('s3').Bucket(self.bucket_name)
        for obj in bucket.objects.filter(Prefix=s3_dir_path):
            local_obj_path = os.path.join(local_dir_path, obj.key)
            if not os.path.exists(os.path.dirname(local_obj_path)):
                os.makedirs(os.path.dirname(local_obj_path))
            bucket.download_file(obj.key, local_obj_path)
            self.logger.info(f'download : {obj.key} to Target: {local_obj_path} Success.')

    def delete_dir(self, s3_dir_path: str):
        '''
        Delete s3 directory.

        :param s3_dir_path: s3 path. ex) nylon-detector/crawl_data
        :return:
        '''
        files = self.list_objects(s3_dir_path)
        if not files:
            return
        s3_keys = []
        for k in self.list_objects(s3_dir_path):
            s3_keys.append(k['Key'])
            if len(s3_keys) % 1000 == 0:
                self.logger.info(f'delete 1000 keys.')
                self.delete_objects(s3_keys)
                s3_keys = []
        if len(s3_keys) > 0:
            self.logger.info(f'delete {len(s3_keys)} keys.')
            self.delete_objects(s3_keys)
        self.logger.info(f'delete {s3_dir_path}')

    def download_file(self, s3_file_path: str, local_file_path: str):
        '''
        Download file from s3.
        \
        :param s3_dir_path: s3 path. ex) nylon-detector/crawl_data/a.csv
        :param local_file_path: local file path. ex) /Users/lks21c/repo/sli-aflow/a.csv
        :return: response
        '''
        bucket = boto3.resource('s3').Bucket(self.bucket_name)
        bucket.download_file(s3_file_path, local_file_path)
        self.logger.info(f'download : {s3_file_path} to Target: {local_file_path} Success.')

    def list_objects(self, prefix: str = '', delimiter: str = ''):
        '''
        List S3 objects.

        :param prefix: Limits the response to keys that begin with the specified prefix.
        :param delimiter: A delimiter is a character you use to group keys.
        :return: response
        '''
        response = self.cli.list_objects_v2(Bucket=self.bucket_name, Prefix=prefix, Delimiter=delimiter)
        return response['Contents'] if 'Contents' in response else None

    def list_dir(self, prefix: str = '', delimiter: str = '/'):
        '''
        List directory.

        :param prefix: Limits the response to keys that begin with the specified prefix.
        :param delimiter: A delimiter is a character you use to group keys.
        :return: response
        '''
        response = self.cli.list_objects_v2(Bucket=self.bucket_name, Prefix=prefix, Delimiter=delimiter)
        list = []
        if 'Contents' in response:
            list += [item['Key'] for item in response['Contents'] if 'Key' in item]
        if 'CommonPrefixes' in response:
            list += [prefix['Prefix'] for prefix in response['CommonPrefixes']]
        return list

    def get_s3_arn(self, bucket_name):
        '''

        :param bucket_name: bucket name
        :return: s3 bucket arn
        '''
        return next(
            (f'arn:aws:s3:::{i["Name"]}' for i in self.list_buckets() if bucket_name in i['Name']),
            None)

    def get_bucket_encryption(self):
        '''

        :return: KMS ID
        '''
        conf = self.cli.get_bucket_encryption(Bucket=self.bucket_name)['ServerSideEncryptionConfiguration']
        return conf['Rules'][0]['ApplyServerSideEncryptionByDefault'] if conf else None



import os

import ujson as json

from baram.log_manager import LogManager


class ConfManager(object):

    @staticmethod
    def load_json_to_dict(filename: str) -> dict:
        """

        :param filename: json file name
        :return: dict
        """

        logger = LogManager.get_logger()

        conf_dir_path = os.path.join(os.getcwd(), '..', 'conf') \
            if 'tests' in os.getcwd() \
            else os.path.join(os.getcwd(), 'conf')
        try:
            with open(os.path.join(conf_dir_path, filename)) as f:
                return json.load(f)
        except Exception as e:
            logger.info(f'error {e}')



import requests

from baram.log_manager import LogManager


class RequestsManager(object):

    @staticmethod
    def get(url: str,
            params: dict = None,
            cookies: dict = None,
            headers: dict = None):
        """

        :param url: url
        :param params: parameters
        :param cookies: cookies
        :param headers: headers
        :return:
        """
        logger = LogManager.get_logger()
        with requests.Session() as s:
            try:
                response = s.get(url, params=params, cookies=cookies, headers=headers)
                return response
            except Exception as e:
                logger.info(e)
